/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'may_open' function
 *
 * Notes:
 *  - Enforce Exploit Detection validation
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 04.X.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_may_open_kretprobe_state = 0x0;

static struct kretprobe p_may_open_kretprobe = {
    .kp.symbol_name = "may_open",
    .handler = p_may_open_ret,
    .entry_handler = p_may_open_entry,
    .data_size = sizeof(struct p_may_open_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_may_open_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_may_open_entry>\n");
   p_debug_kprobe_log(
          "p_may_open_entry: comm[%s] Pid:%d => Arguments: "
          "[path:0x%lx acc_mode:0x%lx flags:0x%lx]\n",
          current->comm,current->pid,p_regs->di,p_regs->si,p_regs->dx);

//   p_ed_enforce_validation();


   spin_lock(&p_rb_ed_pids_lock);
   p_validate_task_f(current);
   spin_unlock(&p_rb_ed_pids_lock);

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_may_open_entry>\n");

   return 0x0;
}


int p_may_open_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_may_open_ret>\n");

//   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_may_open_ret>\n");

   return 0x0;
}


int p_install_may_open_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_may_open_hook>\n");

   if ( (p_ret = register_kretprobe(&p_may_open_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() failed! [err=%d]\n",p_ret);
      goto p_install_may_open_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
                     p_may_open_kretprobe.kp.symbol_name, p_may_open_kretprobe.kp.addr);
   p_may_open_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_may_open_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_may_open_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_may_open_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_may_open_hook>\n");

   if (!p_may_open_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                     p_may_open_kretprobe.kp.symbol_name, p_may_open_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_may_open_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p\n",
                     p_may_open_kretprobe.kp.symbol_name, p_may_open_kretprobe.kp.addr);
      p_may_open_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_may_open_hook>\n");
}
